@@ -631,7 +631,11 @@ def main():
         return False
 
     def getExtraFlags(where, name, flags):
-        if options.assume_yes_for_downloads and name in ("onefile", "standalone"):
+        if options.assume_yes_for_downloads and name in (
+            "onefile",
+            "standalone",
+            "plugins",
+        ):
             yield "--assume-yes-for-downloads"
 
         if os.name == "nt" and options.mingw64:
